#!/usr/bin/env node
// Copyright IBM Corp. 2016. All Rights Reserved.
// Node module: apiconnect
// US Government Users Restricted Rights - Use, duplication or disclosure
// restricted by GSA ADP Schedule Contract with IBM Corp.

var _ = require('lodash');
var format = require('util').format;
var l = require('apiconnect-cli-logger');
var listPlugins = require('apiconnect-cli-plugins').listPlugins;
var path = require('path');
// Load logger first so it can initialize and hook into debug
var logger = require('apiconnect-cli-logger');
var pkg = require('../package.json');

require('../lib/license').licenseAgreed().then(processCmd);

function processCmd(data) {
  var cliPlugins = require('apiconnect-cli-plugins');
  cliPlugins.init(path.resolve(__dirname, path.join('..', 'node_modules')));
  var loadPlugins = cliPlugins.pluginLoader.loadPlugins;

  var buildCli = require('../lib/cli-builder');
  process.argv = _.reject(process.argv, function(arg) { return arg === '--accept-license'; });

  return loadPlugins().then(function(plugins) {
    var program = buildCli(plugins);

    if (!process.argv.slice(2).length) {
      return program.outputHelp();
    } else if (process.argv[2] === '-v' || process.argv[2] === '--version') {
      program.version(pkg.version);
      l.write(pkg.name, ': v');
      program.emit('version');
      return listPlugins();
    } else {
      var notProcessedOptions = [ '-h', '--help' ];
      var validCommands = _.map(program.commands, '._name').concat(notProcessedOptions, program.allAliases);
      var inputCommand = process.argv[2];
      if (validCommands.indexOf(inputCommand) === -1) {
        var errorMsg = format('The command `%s` is not valid', inputCommand);
        var err = new Error(errorMsg);
        err.invalidCommand = true;
        throw err;
      }
      return program.parse(process.argv);
    }
  }).catch(function(err) {
    logger.error(err.message);
    if (err.invalidCommand) {
      return logger.exit(1);
    }
    logger.debug('Error while parsing commandline: %j, %s', process.argv, err.message);
    logger.debug(err.stack);
  });
}
